"""
models.py

App Engine datastore models

"""
# -*- coding: utf-8 -*-
from __future__ import division
from google.appengine.ext import db

from datetime import datetime, date, timedelta

TZ_OFFSET = -7  # change this whenever I change timezones or DST
def get_now():
    now = datetime.utcnow()
    now = now + timedelta(seconds=3600 * (TZ_OFFSET + -4))
    return now

def get_today():
    return get_now().date()

def get_today_datetime():
    today = get_today()
    return datetime(today.year, today.month, today.day)

def get_time_offset_today():
    td = (get_now() - get_today_datetime())
    return td.seconds + td.microseconds / 1E6

class WorkRecord(db.Model):
    """Model for a day's work."""
    date = db.DateProperty(required=True)
    start_times = db.ListProperty(float, default=[], indexed=False)
    stop_times = db.ListProperty(float, default=[], indexed=False)
    category = db.StringProperty(default="Skritter")
           # "Skritter", "CoCo", "nickwinter.net", "QM work", "Telepath"
    project = db.StringProperty(default="iPhone app")
           # "iPhone app", "remnants", "CoCo"
    task = db.StringProperty(default="work")
    synced_to_bodytrack = db.BooleanProperty(default=False)

    def __init__(self, *args, **kargs):
        if not 'date' in kargs:
            kargs['date'] = get_today()
        elif isinstance(kargs['date'], str):
            kargs['date'] = datetime.strptime(kargs['date'], '%Y-%m-%d').date()
        kargs['key_name'] = "%s-%s-%s"%(
                kargs['date'],
                kargs.get('category', 'Skritter'),
                kargs.get('project', 'iOS app'))
        db.Model.__init__(self, *args, **kargs)

    def __repr__(self):       return repr(self.__str__())
    def __str__(self):        return u"<WorkRecord: %s>" % self.date

    @classmethod
    def get(cls, d=None, category="CoCo", project="CoCo"):
        if not d:
            d = get_today()
        wr = cls.get_by_key_name("%s-%s-%s"%(d, category, project))
        if not wr:
            wr = cls(date=d, category=category, project=project)
        return wr

    def start(self):
        if not self.is_working():
            self.start_times.append(get_time_offset_today())
            return True
        return False

    def stop(self):
        if self.is_working():
            self.stop_times.append(get_time_offset_today())
            return True
        return False

    def is_working(self):
        return len(self.start_times) == len(self.stop_times) + 1

    def equals(self, other):
        return (self.date == other.date and
                self.category == other.category and
                self.project == other.project and
                self.start_times == other.start_times and
                self.stop_times == other.stop_times)


"""
views.py
"""

import datetime
import time
import logging

from google.appengine.ext import db

from application import app
from flask import request, render_template, flash, url_for, redirect

import models
from decorators import login_required, admin_required

def coco_stats():
    category, project = 'CoCo', 'CoCo'
    wr = models.WorkRecord.get(category=category, project=project)
    working = wr.is_working()
    refresh_delay = 86400 - models.get_time_offset_today() + 30

    past_wrs = [w for w in models.WorkRecord.all()
                .filter('category =', category)
                .filter('project =', project)
                .fetch(1000)
                if w.date != wr.date]

    return render_template('coco_stats.html', **locals())

@admin_required
def toggle_work(command):
    """Start/stop the work timer."""
    wr = models.WorkRecord.get()
    if command == "start":
        success = wr.start()
    else:
        success = wr.stop()
    if success:
        wr.put()
        return 'success'
    else:
        return 'error'

def update_work():
    """Takes the latest work hour uploads and adjusts the WorkRecords accordingly."""
    new_wrs = pickle.loads(str(request.form.get('work_records', '')))
    if not new_wrs:
        return 'success'
    existing_wrs = models.WorkRecord.all().filter('date >=', new_wrs[0][0]).fetch(9001)
    existing_wrs = [wr for wr in existing_wrs if wr.category == new_wrs[0][1] and wr.project == new_wrs[0][2]]
    for i, wr in enumerate(new_wrs):
        wr = models.WorkRecord(date=wr[0], category=wr[1], project=wr[2],
                               task=wr[3], start_times=wr[4], stop_times=wr[5])
        new_wrs[i] = wr
    new_wrs = [wr for wr in new_wrs
               if not any([wr.equals(other) for other in existing_wrs])]
    logging.info("About to put new WRs: %s"%new_wrs)
    db.put(new_wrs)
    return 'success'

@app.template_filter('jsdate')
def jsdate(value):
    return "new Date(%d, %d, %d)"%(value.year, value.month, value.day)

